@@ -1,5 +1,5 @@
 package Bitcoin::Crypto::Transaction;
-$Bitcoin::Crypto::Transaction::VERSION = '4.000';
+$Bitcoin::Crypto::Transaction::VERSION = '4.001';
 use v5.10;
 use strict;
 use warnings;
@@ -30,6 +30,12 @@ has param 'version' => (
 	default => 1,
 );
 
+has option 'block' => (
+	isa => InstanceOf ['Bitcoin::Crypto::Block'],
+	weak_ref => 1,
+	writer => 1,
+);
+
 has field 'inputs' => (
 	isa => ArrayRef [InstanceOf ['Bitcoin::Crypto::Transaction::Input']],
 	default => sub { [] },
@@ -173,13 +179,23 @@ sub to_serialized
 
 signature_for from_serialized => (
 	method => Str,
-	positional => [ByteStr],
+	head => [ByteStr],
+	named => [
+		pos => Maybe [ScalarRef [PositiveOrZeroInt]],
+		{default => undef},
+	],
+	bless => !!0,
 );
 
 sub from_serialized
 {
-	my ($class, $serialized) = @_;
-	my $pos = 0;
+	my ($class, $serialized, $args) = @_;
+	my $partial = !!$args->{pos};
+	my $pos = $partial ? ${$args->{pos}} : 0;
+
+	# optimization - no need to keep checking bytestrings on every level. It
+	# has already been checked.
+	local $Bitcoin::Crypto::Types::CHECK_BYTESTRINGS = !!0;
 
 	my $version = unpack 'V', substr $serialized, $pos, 4;
 	$pos += 4;
@@ -223,7 +239,10 @@ sub from_serialized
 
 	Bitcoin::Crypto::Exception::Transaction->raise(
 		'serialized transaction data is corrupted'
-	) if $pos != length $serialized;
+	) if !$partial && $pos != length $serialized;
+
+	${$args->{pos}} = $pos
+		if $partial;
 
 	my $tx = $class->new(
 		version => $version,
@@ -400,12 +419,26 @@ sub update_utxos
 			txid => $self->get_hash,
 			output_index => $output_index,
 			output => $output,
+			($self->has_block ? (block => $self->block) : ()),
 		)->register;
 	}
 
 	return $self;
 }
 
+signature_for is_coinbase => (
+	method => Object,
+	positional => [],
+);
+
+sub is_coinbase
+{
+	my ($self) = @_;
+	my $inputs = $self->inputs;
+
+	return @{$inputs} > 0 && $inputs->[0]->utxo_location->[0] eq ("\x00" x 32);
+}
+
 sub _verify_script_default
 {
 	my ($self, $input, $script_runner) = @_;
@@ -602,6 +635,42 @@ sub verify_script
 	);
 }
 
+sub _verify_coinbase
+{
+	my ($self, $block) = @_;
+
+	Bitcoin::Crypto::Exception::Transaction->raise(
+		'coinbase transaction must have one input'
+	) if @{$self->inputs} != 1;
+
+	my $coinbase_data = $self->inputs->[0]->signature_script->to_serialized;
+
+	Bitcoin::Crypto::Exception::Transaction->raise(
+		'coinbase data exceeds 100 bytes'
+	) if length $coinbase_data > 100;
+
+	if (!defined $block) {
+		carp 'trying to verify coinbase transaction but block was not set';
+		return;
+	}
+
+	Bitcoin::Crypto::Exception::Transaction->raise(
+		'coinbase must be the first transaction in a block'
+	) unless $block->transactions->[0] == $self;
+
+	if ($block->version >= 2) {
+		$block->clear_height;
+		Bitcoin::Crypto::Exception::Transaction->raise(
+			'coinbase transaction of version 2 block should contain height'
+		) unless $block->has_height;
+	}
+
+	# NOTE: most of other coinbase verification (like block reward checking)
+	# should probably be made in a block
+
+	return;
+}
+
 signature_for verify => (
 	method => Object,
 	named => [
@@ -614,17 +683,34 @@ signature_for verify => (
 sub verify
 {
 	my ($self, $args) = @_;
-	my $block = $args->{block};
+
+	if ($args->{block}) {
+		carp 'passing block parameter to verify method is deprecated: call set_block on transaction instead';
+		$self->set_block($args->{block});
+	}
+
+	my $block = $self->block;
+	my $inputs = $self->inputs;
+	my $outputs = $self->outputs;
+
+	Bitcoin::Crypto::Exception::Transaction->raise(
+		'transaction has no inputs'
+	) if !@$inputs;
+
+	Bitcoin::Crypto::Exception::Transaction->raise(
+		'transaction has no outputs'
+	) if !@$outputs;
+
+	return $self->_verify_coinbase($block)
+		if $self->is_coinbase;
 
 	my $script_runner = Bitcoin::Crypto::Script::Runner->new(
 		transaction => $self,
 	);
 
-	my @inputs = @{$self->inputs};
-
 	# amount checking
-	my $total_in = sum map { $_->utxo->output->value } @inputs;
-	my $total_out = sum map { $_->value } @{$self->outputs};
+	my $total_in = sum map { $_->utxo->output->value } @$inputs;
+	my $total_out = sum map { $_->value } @$outputs;
 
 	Bitcoin::Crypto::Exception::Transaction->raise(
 		'output value exceeds input'
@@ -634,7 +720,7 @@ sub verify
 	if (
 		$self->locktime > 0 && any {
 			$_->sequence_no != Bitcoin::Crypto::Constants::max_sequence_no
-		} @inputs
+		} @$inputs
 		)
 	{
 		my $locktime = $self->locktime;
@@ -645,18 +731,18 @@ sub verify
 			) if $locktime > ($is_timestamp ? $block->median_time_past : $block->height);
 		}
 		else {
-			carp 'trying to verify locktime but no fitting block parameter was passed';
+			carp 'trying to verify locktime but block was not set';
 		}
 	}
 
 	# per-input verification
-	foreach my $input_index (0 .. $#inputs) {
+	foreach my $input_index (0 .. $#$inputs) {
 		$self->verify_script($input_index, $script_runner);
 
 		# check sequence (BIP 68)
-		if ($self->version >= 2 && !($inputs[$input_index]->sequence_no & (1 << 31))) {
-			my $sequence = $inputs[$input_index]->sequence_no;
-			my $utxo = $inputs[$input_index]->utxo;
+		if ($self->version >= 2 && !($inputs->[$input_index]->sequence_no & (1 << 31))) {
+			my $sequence = $inputs->[$input_index]->sequence_no;
+			my $utxo = $inputs->[$input_index]->utxo;
 			my $time_based = $sequence & (1 << 22);
 			my $relative_locktime = $sequence & 0x0000ffff;
 			my $has_block = defined $block && ($time_based || $block->has_height);
@@ -673,8 +759,7 @@ sub verify
 				) if $now < $then + $relative_locktime;
 			}
 			else {
-				carp
-					'trying to verify relative locktime but no fitting block parameter was passed or utxo block was set';
+				carp 'trying to verify relative locktime but block or utxo block was not set';
 			}
 		}
 	}
@@ -781,6 +866,17 @@ Integer containing locktime of the transaction. By default C<0>.
 
 I<Available in the constructor>.
 
+=head3 block
+
+An optional instance of L<Bitcoin::Crypto::Block>. This reference is weakened.
+Block may be required to do some validations when calling L</verify>.
+
+I<Available in the constructor>.
+
+I<writer:> C<set_block>
+
+I<predicate:> C<has_block>
+
 =head2 Methods
 
 =head3 new
@@ -831,10 +927,24 @@ data. Note that this is a no-op in non-segwit transactions.
 
 =head3 from_serialized
 
-	$object = $class->from_serialized($data)
+	$object = $class->from_serialized($data, %params)
 
 Deserializes the bytestring C<$data> into a transaction object.
 
+C<%params> can be any of:
+
+=over
+
+=item * C<pos>
+
+Position for partial string decoding. Optional. If passed, must be a scalar
+reference to an integer value.
+
+This integer will mark the starting position of C<$bytestring> from which to
+start decoding. It will be set to the next byte after end of transaction stream.
+
+=back
+
 Keep in mind it's best to have a full set of UTXOs registered. If they are not,
 an exception may be raised if a function requires full UTXO data. That
 exception will contain transaction ID and output index, which should help you
@@ -989,12 +1099,23 @@ C<%params> can be any of:
 
 =item * C<block>
 
-Optional instance of L<Bitcoin::Crypto::Block> - used for locktime and sequence
-verification. If it is not passed and the transaction includes these checks, it
-will still verify without an exception but a warning will be issued.
+Optional instance of L<Bitcoin::Crypto::Block> - used for locktime, sequence
+and coinbase verification. If it is not passed and the transaction includes
+these checks, it will still verify without an exception but a warning will be
+issued.
+
+Including this parameter is deprecated - call C<set_block> before verifying instead.
 
 =back
 
+=head3 is_coinbase
+
+	$bool = $object->is_coinbase()
+
+Returns true if this transaction is coinbase: its first input has an empty
+previous transaction hash. Does not check any further - actual validation of
+coinbase is done in L</verify>.
+
 =head3 dump
 
 	$text = $object->dump()
