@@ -1,5 +1,5 @@
 package Bitcoin::Crypto::Util;
-$Bitcoin::Crypto::Util::VERSION = '4.000';
+$Bitcoin::Crypto::Util::VERSION = '4.001';
 use v5.10;
 use strict;
 use warnings;
@@ -343,9 +343,9 @@ sub unpack_compactsize
 	# if the first byte is 0xfd, 0xfe or 0xff, then CompactSize contains 2, 4 or 8
 	# bytes respectively
 	my $value = ord substr $stream, $pos++, 1;
-	my $length = 2**($value - 0xfd + 1);
+	if ($value > 0xfc) {
+		my $length = 1 << ($value - 0xfc);
 
-	if ($length > 1) {
 		Bitcoin::Crypto::Exception->raise(
 			"cannot unpack CompactSize: not enough data in stream"
 		) if length $stream < $length;
@@ -411,6 +411,9 @@ sub merkle_root
 {
 	my ($leaves) = @_;
 
+	# avoid checking bytestrings in (possibly very numerous) hash256 calls
+	local $Bitcoin::Crypto::Types::CHECK_BYTESTRINGS = !!0;
+
 	my @parts = map { hash256($_) } @$leaves;
 
 	Bitcoin::Crypto::Exception->raise(
